<?php
/**
 * Denver Class
 *
 * @author   Themosaurus
 * @since    1.0.0
 * @package  denver
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Denver' ) ) :
	/**
	 * The main Denver class
	 */
	class Denver {
		/**
		 * Setup class.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {
			// Priority 0 to make it available to lower priority callbacks.
			add_action( 'after_setup_theme',  array( $this, 'content_width'      ), 0  );
			add_action( 'after_setup_theme',  array( $this, 'setup'              ), 10 );
			add_action( 'widgets_init',       array( $this, 'widgets_init'       ), 10 );
			add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts' ), 10 );
			add_filter( 'body_class',         array( $this, 'add_body_classes'   ), 10 );
			add_action( 'wp_head',            array( $this, 'pingback_header'    ), 10 );
		}

		/**
		 * Sets up theme defaults and registers support for various WordPress features.
		 *
		 * Note that this function is hooked into the after_setup_theme hook, which
		 * runs before the init hook. The init hook is too late for some features, such
		 * as indicating support for post thumbnails.
		 *
		 * @since 1.0.0
		 */
		public function setup() {
			/*
			 * Make theme available for translation.
			 * Translations can be filed in the /languages/ directory.
			 * If you're building a theme based on denver, use a find and replace
			 * to change 'denver' to the name of your theme in all the template files.
			 */
			load_theme_textdomain( 'denver', get_template_directory() . '/languages' );

			// Add default posts and comments RSS feed links to head.
			add_theme_support( 'automatic-feed-links' );

			/*
			 * Let WordPress manage the document title.
			 * By adding theme support, we declare that this theme does not use a
			 * hard-coded <title> tag in the document head, and expect WordPress to
			 * provide it for us.
			 */
			add_theme_support( 'title-tag' );

			/*
			 * Enable support for Post Thumbnails on posts and pages.
			 *
			 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
			 */
			add_theme_support( 'post-thumbnails' );

			// This theme uses wp_nav_menu() in one location.
			register_nav_menus( apply_filters( 'denver_nav_menus', array(
				'primary' => esc_html__( 'Primary', 'denver' ),
			) ) );

			/*
			 * Switch default core markup for search form, comment form, and comments
			 * to output valid HTML5.
			 */
			add_theme_support( 'html5', array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			) );

			/*
			 * Enable support for Post Formats.
			 *
			 * @link https://developer.wordpress.org/themes/functionality/post-formats/
			 */
			add_theme_support( 'post-formats', apply_filters( 'denver_post_formats_args', array(
				'aside',
				'image',
				'gallery',
				'quote',
				'link',
				'video',
				'audio',
				'status',
				'chat',
			) ) );

			// Set up the WordPress core custom background feature.
			add_theme_support( 'custom-background', apply_filters( 'denver_custom_background_args', array(
				'default-image' => '',
				'default-color' => 'ffffff',
			) ) );

			/**
			 * Set up the WordPress core custom header feature.
			 *
			 * @link https://developer.wordpress.org/themes/functionality/custom-headers/
			 */
			add_theme_support( 'custom-header', apply_filters( 'denver_custom_header_args', array(
				'default-image' => '',
				'header-text'   => false,
				'width'         => 1800,
				'height'        => 650,
				'flex-width'    => true,
				'flex-height'   => true,
			) ) );

			/*
			 * Enable support for custom logo.
			 */
			add_theme_support( 'custom-logo', apply_filters( 'denver_custom_logo_args', array(
				'width'       => 250,
				'height'      => 250,
				'flex-height' => true,
			) ) );

			/**
			 * Add starter content to the theme.
			 */
			add_theme_support( 'starter-content', apply_filters( 'denver_starter_content_args', array() ) );

			/*
			 * Enable support for Woocommerce plugin
			 * See https://docs.woothemes.com/document/declare-woocommerce-support-in-third-party-theme/
			 */
			add_theme_support( 'woocommerce' );

			// Indicate widget sidebars can use selective refresh in the Customizer.
			add_theme_support( 'customize-selective-refresh-widgets' );
		}

		/**
		 * Set the content width in pixels, based on the theme's design and stylesheet.
		 *
		 * @global int $content_width
		 * @since 1.0.0
		 */
		public function content_width() {
			$GLOBALS['content_width'] = apply_filters( 'denver_content_width', 1140 );
		}

		/**
		 * Register widget areas.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
		 * @since 1.0.0
		 */
		public function widgets_init() {
			$widget_areas = apply_filters( 'denver_widget_areas' ,array(
				'sidebar-1' => esc_html__( 'Sidebar 1', 'denver' ),
				'sidebar-2' => esc_html__( 'Sidebar 2', 'denver' ),
			) );

			foreach ( $widget_areas as $widget_area_id => $widget_area_name ) {
				register_sidebar( array(
					'name'          => $widget_area_name,
					'id'            => $widget_area_id,
					'description'   => '',
					'before_widget' => '<section id="%1$s" class="widget %2$s">',
					'after_widget'  => '</section>',
					'before_title'  => '<h2 class="widget-title">',
					'after_title'   => '</h2>',
				) );
			}
		}

		/**
		 * Adds custom classes to the array of body classes.
		 *
		 * @param array $classes Classes for the body element.
		 * @since 1.0.0
		 *
		 * @return array
		 */
		public function add_body_classes( $classes ) {
			// Adds a class of group-blog to blogs with more than 1 published author.
			if ( is_multi_author() ) {
				$classes[] = 'group-blog';
			}

			// Adds a class of hfeed to non-singular pages.
			if ( ! is_singular() ) {
				$classes[] = 'hfeed';
			}

			// Adds a class of mobile to mobile devices pages.
			// @codingStandardsIgnoreLine
			if ( ! apply_filters( 'denver_is_mobile', wp_is_mobile() ) ) {
				$classes[] = 'mobile';
			}
			return $classes;
		}

		/**
		 * Enqueue scripts and stylesheets.
		 *
		 * @since 1.0.0
		 */
		public function wp_enqueue_scripts() {
			/**
			 * Enqueue styles.
			 */
			wp_enqueue_style( 'denver-style', get_stylesheet_uri() );

			/**
			 * Enqueue scripts.
			 */
			wp_enqueue_script( 'denver-skip-link-focus-fix', get_template_directory_uri() . '/assets/js/skip-link-focus-fix.js', array(), '20130115', true );

			if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
				wp_enqueue_script( 'comment-reply' );
			}

			wp_enqueue_script( 'modernizr', get_template_directory_uri() . '/assets/js/libs/modernizr.js', array(), '3.6.0', false );
			wp_enqueue_script( 'popper',    get_template_directory_uri() . '/assets/js/vendor/popper.min.js', array(), '1.12.5', true );
			wp_enqueue_script( 'bootstrap', get_template_directory_uri() . '/assets/js/vendor/bootstrap.min.js', array( 'jquery', 'popper' ), '4.1.3', true );
			wp_enqueue_script( 'denver',    get_template_directory_uri() . '/assets/js/main.js', array( 'bootstrap' ), DENVER_VERSION, true );
		}

		/**
		 * Add a pingback url auto-discovery header for singularly identifiable articles.
		 *
		 * @since 1.0.0
		 */
		public function pingback_header() {
			if ( is_singular() && pings_open() ) {
				echo '<link rel="pingback" href="', esc_url( get_bloginfo( 'pingback_url' ) ), '">';
			}
		}
	}
endif;

return new Denver();
