<?php
/**
 * Denver Grimlock Class
 *
 * @package  denver
 * @author   Themosaurus
 * @since    1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Denver_Grimlock' ) ) :
	/**
	 * The Denver Grimlock integration class
	 */
	class Denver_Grimlock {
		/**
		 * Setup class.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {
			add_action( 'after_setup_theme',                                      array( $this, 'add_image_sizes'                                      ), 10    );
			add_filter( 'image_size_names_choose',                                array( $this, 'add_image_size_names'                                 ), 20, 1 );
			add_action( 'widgets_init',                                           array( $this, 'widgets_init'                                         ), 10    );
			add_action( 'init',                                                   array( $this, 'add_excerpt_support_for_pages'                        ), 10    );
			add_filter( 'denver_widget_areas',                                    array( $this, 'add_widget_areas'                                     ), 10, 1 );
			add_filter( 'grimlock_content_class',                                 array( $this, 'add_content_classes'                                  ), 10, 1 );
			add_filter( 'grimlock_posts_section_widget_post_thumbnail_size',      array( $this, 'change_posts_section_widget_post_thumbnail_size'      ), 10, 3 );
			add_filter( 'grimlock_query_section_widget_post_thumbnail_size',      array( $this, 'change_posts_section_widget_post_thumbnail_size'      ), 10, 3 );
			add_filter( 'grimlock_term_query_section_widget_term_thumbnail_size', array( $this, 'change_term_query_section_widget_term_thumbnail_size' ), 10, 3 );
			add_filter( 'grimlock_custom_header_displayed',                       array( $this, 'has_custom_header_displayed'                          ), 10, 1 );
		}

		/**
		 * Add image sizes for use in different section layouts
		 */
		public function add_image_sizes() {
			$sizes = apply_filters( 'denver_grimlock_image_sizes', array(
				'thumbnail-12-cols-classic'      => array(
					'width'  => 1280,
					'height' => 600,
					'crop'   => true,
				),
				'thumbnail-6-6-cols-classic'     => array(
					'width'  => 700,
					'height' => 400,
					'crop'   => true,
				),
				'thumbnail-4-4-4-cols-classic'   => array(
					'width'  => 700,
					'height' => 500,
					'crop'   => true,
				),
				'thumbnail-3-3-3-3-cols-classic' => array(
					'width'  => 550,
					'height' => 390,
					'crop'   => true,
				),
				'thumbnail-12-cols-lateral'      => array(
					'width'  => 700,
					'height' => 520,
					'crop'   => true,
				),
				'thumbnail-6-6-cols-lateral'     => array(
					'width'  => 550,
					'height' => 650,
					'crop'   => true,
				),
			) );

			foreach ( $sizes as $name => $size ) {
				add_image_size( $name, $size['width'], $size['height'], $size['crop'] );
			}
		}

		/**
		 * Define names for the custom image sizes.
		 *
		 * @param array $sizes The array of custom sizes.
		 *
		 * @return array The array of custom sizes.
		 * @since 1.0.1
		 */
		public function add_image_size_names( $sizes ) {
			return array_merge( $sizes, array(
				'thumbnail-12-cols-classic'      => esc_html__( 'Thumbnail: One Column', 'denver' ),
				'thumbnail-6-6-cols-classic'     => esc_html__( 'Thumbnail: Two Columns', 'denver' ),
				'thumbnail-4-4-4-cols-classic'   => esc_html__( 'Thumbnail: Three Columns', 'denver' ),
				'thumbnail-3-3-3-3-cols-classic' => esc_html__( 'Thumbnail: Four Columns', 'denver' ),
				'thumbnail-12-cols-lateral'      => esc_html__( 'Thumbnail: One Column Lateral', 'denver' ),
				'thumbnail-6-6-cols-lateral'     => esc_html__( 'Thumbnail: Two Columns Lateral', 'denver' ),
			) );
		}

		/**
		 * Enables the Excerpt meta box in Page edit screen.
		 *
		 * @since 1.0.0
		 */
		public function add_excerpt_support_for_pages() {
			add_post_type_support( 'page', 'excerpt' );
		}

		/**
		 * Register widget areas for the front page.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
		 * @since 1.0.0
		 */
		public function widgets_init() {
			$widget_areas = apply_filters( 'denver_grimlock_widget_areas', array(
				'homepage-1'       => esc_html__( 'Homepage', 'denver' ),
				'before-content-1' => esc_html__( 'Before Content', 'denver' ),
				'after-content-1'  => esc_html__( 'After Content', 'denver' ),
			) );

			foreach ( $widget_areas as $widget_area_id => $widget_area_name ) {
				register_sidebar( array(
					'name'          => $widget_area_name,
					'id'            => $widget_area_id,
					'description'   => '',
					'before_widget' => '<section id="%1$s" class="widget %2$s">',
					'after_widget'  => '</section>',
					'before_title'  => '<span class="widget-title">',
					'after_title'   => '</span>',
				) );
			}
		}

		/**
		 * Add widget areas for the theme.
		 *
		 * @since 1.0.0
		 *
		 * @param  array $widget_areas The array of widget areas to register.
		 *
		 * @return array               The filtred array of widget areas to register.
		 */
		public function add_widget_areas( $widget_areas ) {
			return array_merge( $widget_areas, array(
				'preheader-1' => esc_html__( 'Pre Header 1', 'denver' ),
				'preheader-2' => esc_html__( 'Pre Header 2', 'denver' ),
				'preheader-3' => esc_html__( 'Pre Header 3', 'denver' ),
				'preheader-4' => esc_html__( 'Pre Header 4', 'denver' ),
				'prefooter-1' => esc_html__( 'Pre Footer 1', 'denver' ),
				'prefooter-2' => esc_html__( 'Pre Footer 2', 'denver' ),
				'prefooter-3' => esc_html__( 'Pre Footer 3', 'denver' ),
				'prefooter-4' => esc_html__( 'Pre Footer 4', 'denver' ),
				'footer-1'    => esc_html__( 'Footer 1', 'denver' ),
				'footer-2'    => esc_html__( 'Footer 2', 'denver' ),
				'footer-3'    => esc_html__( 'Footer 3', 'denver' ),
				'footer-4'    => esc_html__( 'Footer 4', 'denver' ),
			) );
		}

		/**
		 * Add custom classes to content to modify layout.
		 *
		 * @since 1.0.0
		 *
		 * @param  array $classes The array of CSS classes to apply to the `#content` element.
		 *
		 * @return array $classes The filtered array of CSS classes to apply to the `#content` element.
		 */
		public function add_content_classes( $classes ) {
			if ( is_page() ) {
				$page_template = apply_filters( 'denver_grimlock_page_template', get_page_template_slug( get_queried_object_id() ) );

				switch ( $page_template ) {
					case 'template-narrower-12-cols-left.php':
						$classes[] = 'region--12-cols-left';
						$classes[] = 'region--container-narrower';
						break;

					case 'template-narrow-12-cols-left.php':
						$classes[] = 'region--12-cols-left';
						$classes[] = 'region--container-narrow';
						break;

					case 'template-classic-12-cols-left.php':
						$classes[] = 'region--12-cols-left';
						$classes[] = 'region--container-classic';
						break;

					case 'template-classic-9-3-cols-left.php':
						$classes[] = 'region--9-3-cols-left';
						$classes[] = 'region--container-classic';
						break;

					case 'template-classic-3-9-cols-left.php':
						$classes[] = 'region--3-9-cols-left';
						$classes[] = 'region--container-classic';
						break;

					case '': // Default Template.
						$classes[] = 'region--3-6-3-cols-left';
						$classes[] = 'region--container-classic';
				}
			}
			return $classes;
		}

		/**
		 * Change the post thumbnail size for custom WP queries.
		 *
		 * @since 1.0.1
		 *
		 * @param string $size         The post thumbnail size.
		 * @param string $posts_layout The layout for posts.
		 * @param string $post_type    The post type.
		 *
		 * @return string              The changed post thumbnail size.
		 */
		public function change_posts_section_widget_post_thumbnail_size( $size, $posts_layout, $post_type ) {
			return "thumbnail-{$posts_layout}";
		}

		/**
		 * Change default term thumbnail sizes for the query section widget.
		 *
		 * @since 1.0.1
		 *
		 * @param string $size         The size for the term thumbnail.
		 * @param string $terms_layout The layout for the term query.
		 * @param string $taxonomy     The taxonomy name.
		 *
		 * @return string              The updated size for the term thumbnail.
		 */
		public function change_term_query_section_widget_term_thumbnail_size( $size, $terms_layout, $taxonomy ) {
			return "thumbnail-{$terms_layout}";
		}

		/**
		 * Check if the custom header is displayed or not.
		 *
		 * @since 1.0.1
		 *
		 * @param bool $default True if the custom header would be displayed, false otherwise.
		 *
		 * @return bool True if the custom header is displayed, false otherwise.
		 */
		public function has_custom_header_displayed( $default ) {
			return ! is_page_template( 'template-homepage.php' ) && $default;
		}
	}
endif;

return new Denver_Grimlock();
